require 'modshogun'

f_feats_train = modshogun.CSVFile("../../data/regression_1d_sinc_features_train.dat")
f_feats_test = modshogun.CSVFile("../../data/regression_1d_sinc_features_test.dat")
f_labels_train = modshogun.CSVFile("../../data/regression_1d_sinc_labels_train.dat")
f_labels_test = modshogun.CSVFile("../../data/regression_1d_sinc_labels_test.dat")

--![create_features]
features_train = modshogun.RealFeatures(f_feats_train)
features_test = modshogun.RealFeatures(f_feats_test)
labels_train = modshogun.RegressionLabels(f_labels_train)
labels_test = modshogun.RegressionLabels(f_labels_test)
--![create_features]

--![create_appropriate_kernel_and_mean_function]
width = 1.0
kernel = modshogun.GaussianKernel(features_train, features_train, width)
mean_function = modshogun.ZeroMean()
--![create_appropriate_kernel_and_mean_function]

--![create_instance]
gauss_likelihood = modshogun.GaussianLikelihood()
inference_method = modshogun.ExactInferenceMethod(kernel, features_train, mean_function, labels_train, gauss_likelihood)
gp_regression = modshogun.GaussianProcessRegression(inference_method)
--![create_instance]

--![train_and_apply]
gp_regression:train()
labels_predict = gp_regression:apply_regression(features_test)
--![train_and_apply]

--![compute_variance]
variances = gp_regression:get_variance_vector(features_test)
--![compute_variance]

--![optimize_marginal_likelihood]
grad_criterion = modshogun.GradientCriterion()
grad = modshogun.GradientEvaluation(gp_regression, features_train, labels_train, grad_criterion)
grad:set_function(inference_method)
grad_selection = modshogun.GradientModelSelection(grad)
best_theta = grad_selection:select_model()
best_theta:apply_to_machine(gp_regression)
--![optimize_marginal_likelihood]

--![evaluate_error_and_marginal_likelihood]
eval = modshogun.MeanSquaredError()
mse = eval:evaluate(labels_predict, labels_test)
marg_ll = inference_method:get_negative_log_marginal_likelihood()
--![evaluate_error_and_marginal_likelihood]

-- integration testing variables
output = labels_test:get_labels()

-- Serialize output for integration testing (automatically generated)
__sg_storage = modshogun.WrappedObjectArray()
__sg_storage_file = modshogun.SerializableAsciiFile("gaussian_process_regression.dat", 'w')
__sg_storage:append_wrapped_real(width, "width")
__sg_storage:append_wrapped_real_vector(variances, "variances")
__sg_storage:append_wrapped_real(mse, "mse")
__sg_storage:append_wrapped_real(marg_ll, "marg_ll")
__sg_storage:append_wrapped_real_vector(output, "output")
__sg_storage:save_serializable(__sg_storage_file)
